
/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: REQUISICAO_ANALITICA_SETOR_PLA

Descrio:                                                             
Clculo do Consumo Planejado (Cardpio, Consumo por Servio Fixo ou    
Percapita) atravs da stored procedure CONSUMO_PLANEJADO e gravao na 
tabela PREVRETI.                                                     

P_FILIAL       - Codigo da Filial                                          
P_SERVICOS     - String contendo sequencia de Servios a serem calculados  
P_DTINICIAL    - Data inicial                                              
P_DTFINAL      - Data final                                                
P_IDARQUIVO    - Identificador da tabela a ser gravada                     
                 "P" -> PREVRETI                                          
                 "C" -> CONSPLAN
                 "A" -> AMBAS                                             
P_IDPRODUTOS   - Identificador dos produtos a serem gravados.              
                 Formato "XXX"                                            
                 1 posio = "C" -> Cardpio                             
                 2 posio = "F" -> Consumo por Servio Fixo             
                 3 posio = "P" -> Consumo por Servio Percapita        
P_IDSEPPRATO   - Identificador de requisio analtica, ou seja, separada  
                 por prato ou por setor, consumo fixo e consumo percapita.
                 Formato "S" (setor), "P" (prato) ou "N" (n/a).
P_PEACRESCIMO  - Percentual de acrscimo                                   
P_IDARREDOND   - Identificador de utilizao do arredondamento             
                 para requisio do Produto. Formato "S" ou "N"           
P_NRCONSPLAN   - Nmero sequencial a ser utilizado para gravao dos dados 
P_CDPRODUTOINI - Cdigo do Produto inicial                                 
P_CDPRODUTOFIN - Cdigo do Produto final                                   
P_CDOPERADOR   - Cdigo do Operador
P_DTHRCALCPR   - Data/hora do clculo
P_STDTINICIAL  - Lista de data inicial em string por servio
P_STDTFINAL    - Lista de data final em string por servio

Criao:                                                            
01/03/99 - Ederson
OBS.: Convertida para SQL SERVER por Robert

*/


IF EXISTS(SELECT name FROM sysobjects
          WHERE name = 'REQUISICAO_ANALITICA_SETOR_PLA' AND type = 'P')
  DROP PROCEDURE REQUISICAO_ANALITICA_SETOR_PLA
  GO

CREATE PROCEDURE REQUISICAO_ANALITICA_SETOR_PLA @P_FILIAL       VARCHAR(4),
                                                @P_SERVICOS     VARCHAR(2000),
                                                @P_DTINICIAL    DATETIME,
                                                @P_DTFINAL      DATETIME,
                                                @P_IDARQUIVO    VARCHAR(1),
                                                @P_IDPRODUTOS   VARCHAR(3),
                                                @P_IDSEPPRATO   VARCHAR(1),
                                                @P_PEACRESCIMO  NUMERIC(15,6),
                                                @P_IDARREDOND   VARCHAR(1),
                                                @P_NRCONSPLAN   VARCHAR(10),
                                                @P_CDPRODUTOINI VARCHAR(10),
                                                @P_CDPRODUTOFIN VARCHAR(10),
                                                @P_CDOPERADOR   VARCHAR(12),
                                                @P_DTHRCALCPR   DATETIME,
                                                @P_STDTINICIAL  VARCHAR(2000),
                                                @P_STDTFINAL    VARCHAR(2000)


AS

   DECLARE @V_CDSERVFILI   VARCHAR(5);
   DECLARE @V_CDSERVFILIAUX VARCHAR(4000)
   DECLARE @V_CDPRODUTOFIN VARCHAR(10);
   DECLARE @V_CDPRODFINAUX VARCHAR(20);
   DECLARE @V_CDFILIAL     VARCHAR(4);

BEGIN
  /* REGISTRA O CALCULO NA TABELA DE LOG */

   IF (@P_IDARQUIVO <> 'C')
   BEGIN
   SET @V_CDSERVFILIAUX = '';
      DECLARE C_FILIAIS CURSOR FOR
      SELECT DISTINCT CDFILIAL
        FROM SERVICO
       WHERE (CDFILIAL = @P_FILIAL)          
       UNION ALL     
      SELECT DISTINCT SUBSTRING(CDLISTA, 1, 4) AS CDFILIAL
        FROM LISTA
       WHERE NRLISTA = @P_SERVICOS;  

      OPEN C_FILIAIS;
      FETCH NEXT FROM C_FILIAIS INTO @V_CDFILIAL;
      WHILE (@@FETCH_STATUS = 0)
      BEGIN
         DECLARE C_SERVICO CURSOR FOR(             
             SELECT CDSERVFILI 
               FROM SERVICO
              WHERE CDFILIAL = @V_CDFILIAL
                AND (CHARINDEX(CDSERVFILI, @P_SERVICOS) > 0 OR @P_SERVICOS = 'T')
                AND NOT EXISTS (SELECT SUBSTRING(CDLISTA, 5, 5) AS CDSERVFILI
                                  FROM LISTA
                                 WHERE NRLISTA = @P_SERVICOS 
                                   AND SUBSTRING(CDLISTA, 1, 4) = @V_CDFILIAL)
              UNION       
             SELECT SUBSTRING(CDLISTA, 5, 5) AS CDSERVFILI
               FROM LISTA
              WHERE NRLISTA = @P_SERVICOS 
                AND SUBSTRING(CDLISTA, 1, 4) = @V_CDFILIAL)
         OPEN C_SERVICO;
         FETCH NEXT FROM C_SERVICO INTO @V_CDSERVFILI;

         WHILE (@@FETCH_STATUS = 0)
         BEGIN   
            INSERT INTO LOGPREVRETI
              (CDFILIAL,
               CDOPERADOR,
               DTHRCALCPR,
               NRCONSPLANPR,
               CDSERVFILI,
               DTINICIALPR,
               DTFINALPR,
               CDPRODINICPR,
               CDPRODFINAPR,
               PEACRESCIMPR)
            VALUES
              (@V_CDFILIAL,
               @P_CDOPERADOR,
               @P_DTHRCALCPR,
               @P_NRCONSPLAN,
               @V_CDSERVFILI,
               @P_DTINICIAL,
               @P_DTFINAL,
               @P_CDPRODUTOINI,
               @P_CDPRODUTOFIN,
               @P_PEACRESCIMO);

            SET @V_CDSERVFILIAUX = @V_CDSERVFILIAUX + '_' + @V_CDSERVFILI;
            FETCH NEXT FROM C_SERVICO INTO @V_CDSERVFILI;
          
         END;
         CLOSE C_SERVICO;
         DEALLOCATE C_SERVICO;  


         SET @V_CDPRODUTOFIN = @P_CDPRODUTOFIN;
  
         SET @V_CDPRODFINAUX = SUBSTRING(@V_CDPRODUTOFIN+REPLICATE('z',10),1,10);

         EXEC CONSUMO_PLANEJADO_PLA @V_CDFILIAL,
                                    @V_CDSERVFILIAUX,
                                    @P_DTINICIAL,
                                    @P_DTFINAL,
                                    @P_IDARQUIVO,
                                    @P_IDPRODUTOS,
                                    @P_IDSEPPRATO,
                                    @P_PEACRESCIMO,
                                    @P_IDARREDOND,
                                    @P_NRCONSPLAN,
                                    @P_CDPRODUTOINI,
                                    @V_CDPRODFINAUX,
                                    @P_CDOPERADOR,
                                    @P_STDTINICIAL,
                                    @P_STDTFINAL;
         SET @V_CDSERVFILIAUX = '';
         SET @V_CDFILIAL = '';

         FETCH NEXT FROM C_FILIAIS INTO @V_CDFILIAL;
      END; /* LOOP DOS SERVIOS */

      CLOSE C_FILIAIS;
      DEALLOCATE C_FILIAIS;
   END;
END;


GO

-- VERSAO --
EXEC ATUVEROBJ 'REQUISICAO_ANALITICA_SETOR_PLA','5.05.014';
GO

