const fs = require('fs');

let profile = process.argv.slice(2)[0];
 
// TODO Verificar possibilidade customizar paths frontend/config

let frontendPath = __dirname + '/../frontend/';
let pathConfig = frontendPath + 'src/config/';
let configFilePath = pathConfig + 'devConfig.json';


const readConfig = ( filesToRead ) => {// {ENV: '/path/to/env', ENVJSON: '/path/to/envjson/}

    try {
        const FILES = Object.keys(filesToRead);
        const FILES_LOADED = {};
        FILES.forEach( nameProp => {
            let currentPath = filesToRead[nameProp];
            let configStr = fs.readFileSync(currentPath, 'utf8');
            FILES_LOADED[nameProp] = handleFileExtension( currentPath, configStr );
        });
        return FILES_LOADED;
    } catch( error ) {
        console.error("Falha na leitura de arquivo de configuração", error);
    }
    return null;
}

const handleFileExtension = ( filePath, fileContent ) => {
    try {
        if(!filePath || typeof fileContent !== 'string') return fileContent;
        const EXTENSION_PATTERN = /\.\w+$/gi;
        const [ EXTENSION ] = filePath.match(EXTENSION_PATTERN) || [];
        if(!EXTENSION) return fileContent;
        
        switch( EXTENSION.toLowerCase() ){
            case '.json': return JSON.parse(fileContent);
        }
    } catch( error ) {
        console.log( error );
    }
    return fileContent;
}



const handleModules = (productPath, configModules) => {
    Object.keys(configModules).forEach( moduleName => {

    });
}

let getMnemonics = ( string ) => {
    const PATTERN = /(?<START><>)(?<PRODUCT>\w+)\.(?<MNEMONIC>\w+)(?<END><>)/gi;
    return [...string.matchAll(PATTERN)].reduce( (acc, cv) => {
        let [ MNEMONIC ] = cv;
        MNEMONIC = MNEMONIC.replace(/<>/gi, '');
        acc.push(MNEMONIC);
        return acc;
    }, []);
    
};

let handleSpecificMnemonics = (PRODUCT_NAME, MNEMONIC, ALL_MNEMONICS) => {
    const { PROTOCOL, IP, EXTERNAL_PORT, EXTERNAL_PORT_80, PORT} = Object.assign({}, ALL_MNEMONICS, ALL_MNEMONICS[PRODUCT_NAME] || {});
    switch( MNEMONIC.toUpperCase() ) {
        case 'URL_BACKEND': 
            return `${PROTOCOL}://${IP}:${EXTERNAL_PORT_80}/`;
        case 'URL_FRONTEND': 
            return `${PROTOCOL}://${IP}:${EXTERNAL_PORT}/`;
        case 'PORT':
            return PORT;
    }
    return 'EMPTY';
}

const validateConfigJson = ( devConfigJson, currentProfile ) => {
    if(!currentProfile){
        throw Error("Um profile deve ser informado");
    } else if(!devConfigJson){
        throw Error("Falha na estrutura do arquivo de configuração");
    } else if(!devConfigJson.PROFILES){
        throw Error("Arquivo de configuração não possui profile definido");
    } else if(typeof devConfigJson.PROFILES !== 'object' || Array.isArray(devConfigJson.PROFILES)){
        throw Error("( PROFILES ) deve ser definido como um objeto");
    } else if(!devConfigJson.PROFILES[currentProfile]) {
        throw Error(`Não há configuração parametrizada para o profile -- ${currentProfile}`)
    }
    return true;
}

const isJson = ( filePath ) => {
    return filePath && filePath.match(/\.json$/gi);
}

const writeConfigFile = (basePath, productName, profileConfig) => {
    const FRONT_PATH = `${basePath}/${productName}/frontend/`;
    const CONFIG_PATH = `${FRONT_PATH}/src/config`;
    const FILES_TO_VERIFY = {
        ENV: `${FRONT_PATH}/.env.example`, 
        DEV_ENV: `${CONFIG_PATH}/devEnvironment.json.example`
    };
    const FILES_STR = readConfig( FILES_TO_VERIFY );
    if(FILES_STR) {
        Object.keys(FILES_TO_VERIFY).forEach( aliasFile => {
            let newFileName = FILES_TO_VERIFY[aliasFile].replace(/\b\.example\b$/gi, '');
            let newStr = replaceAllMnemonics(FILES_STR[aliasFile], profileConfig);
            let params = [newFileName, newStr];
            if(isJson(newFileName)) {
                params = params.concat([null, 4]);
            }
            fs.writeFileSync(...params);
        });
    } else {
        console.error(`Erro ao ler arquivos .example do produto/modulo ${productName}`);
    }
}

const handleConfigFiles = ( devConfigJson, selectedProfile ) => {
    if( validateConfigJson(devConfigJson, selectedProfile) ) {
        const { DEFAULTS } = devConfigJson;
        const {CUSTOM_CONFIG = {}, MODULES, ...OTHER_PROPS} = devConfigJson.PROFILES[profile];
        let profileConfig = Object.assign({}, DEFAULTS, CUSTOM_CONFIG, OTHER_PROPS);
        let {MODULES: _, ...PRODUCT_PROPS} = profileConfig.PRODUCT;
        let basePathProduct = `${profileConfig.BASE_PATH}/${profileConfig.PRODUCT.NAME_FOLDER}`;
        
        writeConfigFile(profileConfig.BASE_PATH, profileConfig.PRODUCT.NAME_FOLDER, profileConfig);
        if( MODULES ) {
            Object.keys( MODULES ).forEach( moduleName => {
                let profileModuleConfig = Object.assign({}, DEFAULTS, CUSTOM_CONFIG, MODULES[moduleName]);
                let moduleFolder = profileModuleConfig.NAME_FOLDER || moduleName; 
                let basePathModule = `${profileConfig.BASE_PATH}/${profileConfig.NAME_FOLDER}/modules/${moduleFolder}`;
                writeConfigFile(basePathModule, moduleFolder, profileModuleConfig);
            });
        }
    }
};

function replaceAllMnemonics(strToReplace, profileConfig) {
    
    let mnemonicsInFile = getMnemonics(strToReplace, profileConfig);
    let newStr = strToReplace;
    if(Array.isArray(mnemonicsInFile)) {
        mnemonicsInFile.forEach( mnemonic => {
            let [ PRODUCT_NAME, REAL_MNEMONIC ] = mnemonic.split('.');
             const ALL_MNEMONICS = Object.assign({}, profileConfig, profileConfig.PRODUCT, profileConfig.PRODUCT.MODULES);
             let newValue = ALL_MNEMONICS[REAL_MNEMONIC];
             let specificValue = handleSpecificMnemonics(PRODUCT_NAME, REAL_MNEMONIC, ALL_MNEMONICS);
             if(specificValue !== 'EMPTY') {
                 newValue = specificValue;
             } else {
                 console.log('Não foi encontrado valor para o mnemonico.');
             }
             
             newStr = newStr.replace(`<>${mnemonic}<>`, newValue);
        });
        return newStr;
    }
}

const {DEV_CONFIG} = readConfig({DEV_CONFIG: configFilePath}) || {};
handleConfigFiles(DEV_CONFIG, profile);